##############################################################################
# 
# Zope Public License (ZPL) Version 1.0
# -------------------------------------
# 
# Copyright (c) Digital Creations.  All rights reserved.
# 
# This license has been certified as Open Source(tm).
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
# 
# 1. Redistributions in source code must retain the above copyright
#    notice, this list of conditions, and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions, and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
# 
# 3. Digital Creations requests that attribution be given to Zope
#    in any manner possible. Zope includes a "Powered by Zope"
#    button that is installed by default. While it is not a license
#    violation to remove this button, it is requested that the
#    attribution remain. A significant investment has been put
#    into Zope, and this effort will continue if the Zope community
#    continues to grow. This is one way to assure that growth.
# 
# 4. All advertising materials and documentation mentioning
#    features derived from or use of this software must display
#    the following acknowledgement:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    In the event that the product being advertised includes an
#    intact Zope distribution (with copyright and license included)
#    then this clause is waived.
# 
# 5. Names associated with Zope or Digital Creations must not be used to
#    endorse or promote products derived from this software without
#    prior written permission from Digital Creations.
# 
# 6. Modified redistributions of any form whatsoever must retain
#    the following acknowledgment:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    Intact (re-)distributions of any official Zope release do not
#    require an external acknowledgement.
# 
# 7. Modifications are encouraged but must be packaged separately as
#    patches to official Zope releases.  Distributions that do not
#    clearly separate the patches from the original work must be clearly
#    labeled as unofficial distributions.  Modifications which do not
#    carry the name Zope may be packaged in any form, as long as they
#    conform to all of the clauses above.
# 
# 
# Disclaimer
# 
#   THIS SOFTWARE IS PROVIDED BY DIGITAL CREATIONS ``AS IS'' AND ANY
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL DIGITAL CREATIONS OR ITS
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
#   SUCH DAMAGE.
# 
# 
# This software consists of contributions made by Digital Creations and
# many individuals on behalf of Digital Creations.  Specific
# attributions are listed in the accompanying credits file.
# 
##############################################################################

from string import strip, split, join
import sy_occ, string, DateTime
from sy_occ import CS_SET, CS_SUCCEED

from zLOG import LOG, ERROR, PROBLEM
from Shared.DC.ZRDB.TM import TM
import Missing
MissingValue=Missing.Value
del Missing

_context=sy_occ.cs_ctx_alloc()

_string_types={
    sy_occ.CS_CHAR_TYPE: 's',
    sy_occ.CS_LONGCHAR_TYPE: 's',
    sy_occ.CS_BINARY_TYPE: 's',
    sy_occ.CS_LONGBINARY_TYPE: 's',
    sy_occ.CS_TEXT_TYPE: 's',
    sy_occ.CS_IMAGE_TYPE: 's',
    }

_type_map={    
    sy_occ.CS_DATETIME_TYPE:'d',
    sy_occ.CS_DATETIME4_TYPE:'d',
    sy_occ.CS_BIT_TYPE:'i',
    sy_occ.CS_TINYINT_TYPE:'i',
    sy_occ.CS_SMALLINT_TYPE:'i',
    sy_occ.CS_INT_TYPE:'i',
    sy_occ.CS_NUMERIC_TYPE:'i',
    sy_occ.CS_DECIMAL_TYPE:'i',
    sy_occ.CS_FLOAT_TYPE:'n',
    sy_occ.CS_REAL_TYPE:'n',
    sy_occ.CS_MONEY_TYPE:'n',
    sy_occ.CS_MONEY4_TYPE:'n',
    }
_type_map.update(_string_types)

converters={
    sy_occ.CS_DATETIME_TYPE:  DateTime.DateTime,
    sy_occ.CS_DATETIME4_TYPE: DateTime.DateTime,
    }

class Error(Exception): pass

hack=[]
    
class DB(TM):

    def __init__(self, connection):
        server=connection.server
        user=connection.user
        password=connection.password
        database=getattr(connection, 'database', '')
        self._clean=getattr(connection, 'transactional', '')

        db=_context.ct_con_alloc()
        db.ct_diag(sy_occ.CS_INIT)

        if ( db.ct_con_props(CS_SET, sy_occ.CS_USERNAME, user)
             != CS_SUCCEED ): self._error()
        if ( db.ct_con_props(CS_SET, sy_occ.CS_PASSWORD, password)
             != CS_SUCCEED ): self._error()

        if db.ct_connect(server) != CS_SUCCEED: self._error()

        self._db=db
        self._cm=db.ct_cmd_alloc()

        if database: self.query("use %s" % database, simple=1)

        self._warning=self._warn() # clear spurious message

    def close(self): self._db=self._cm=None

    def query(self, query_string, max_rows=None, simple=0,
              # Static vars
              DATAFMT=sy_occ.DATAFMT,
              CS_LANG_CMD=sy_occ.CS_LANG_CMD,
              CS_SUCCEED=sy_occ.CS_SUCCEED,
              CS_END_RESULTS=sy_occ.CS_END_RESULTS,
              CS_ROW_RESULT=sy_occ.CS_ROW_RESULT,
              CS_NUMDATA=sy_occ.CS_NUMDATA,
              CS_CMD_SUCCEED=sy_occ.CS_CMD_SUCCEED,
              CS_CMD_DONE=sy_occ.CS_CMD_DONE,
              CS_CMD_FAIL=sy_occ.CS_CMD_FAIL,
              Buffer=sy_occ.Buffer,
              CS_INT_TYPE=sy_occ.CS_INT_TYPE,
              CS_SMALLINT_TYPE=sy_occ.CS_SMALLINT_TYPE,
              CS_FLOAT_TYPE=sy_occ.CS_FLOAT_TYPE,
              CS_MONEY_TYPE=sy_occ.CS_MONEY_TYPE,
              CS_MONEY4_TYPE=sy_occ.CS_MONEY4_TYPE,
              CS_NUMERIC_TYPE=sy_occ.CS_NUMERIC_TYPE,
              CS_DECIMAL_TYPE=sy_occ.CS_DECIMAL_TYPE,
              CS_TEXT_TYPE=sy_occ.CS_TEXT_TYPE,
              CS_IMAGE_TYPE=sy_occ.CS_IMAGE_TYPE,
              CS_ROW_FAIL=sy_occ.CS_ROW_FAIL,
              CS_END_DATA=sy_occ.CS_END_DATA,
              CS_SUCCEED=sy_occ.CS_SUCCEED,
              CS_FMT_UNUSED=sy_occ.CS_FMT_UNUSED,
              string_type=_string_types.has_key,
              type_map=_type_map,
              rstrip=string.rstrip,
              TupleType=type(()),
              ):

        if self._clean and not simple: self._register()

        cm=self._cm
        result=()
        dd=()

        queries=filter(None, map(strip,split(query_string, '\0')))
        for query in queries:

            failed=0
            if cm.ct_command(CS_LANG_CMD, query) != CS_SUCCEED: self._error()
            if cm.ct_send() != CS_SUCCEED: self._error()

            while 1:

                rc, rt = cm.ct_results()
                if rc==CS_END_RESULTS: break
                
                if rc != CS_SUCCEED: self._error()

                if rt == CS_ROW_RESULT:

                    rc, nv = cm.ct_res_info(CS_NUMDATA)
                    if rc != CS_SUCCEED: self._error()
                    
                    mv=MissingValue

                    if result:
                        cm.ct_cancel(sy_occ.CS_CANCEL_ALL)
                        raise Error("Attempt to use more than one select")

                    result=[]
                    f=DATAFMT()
                    bindings=self._b=range(nv)
                    hack.append(bindings)
                    dd=range(nv)
                    for i in bindings:
                        si=i+1
                        if cm.ct_describe(si, f) != CS_SUCCEED: self._error()

                        if f.datatype in (CS_MONEY_TYPE, CS_MONEY4_TYPE):
                            # Punt on money for now:
                            f.datatype=CS_FLOAT_TYPE
                        elif f.datatype in (CS_NUMERIC_TYPE, CS_DECIMAL_TYPE):
                            # Use float and int for number when we can:
                            if f.scale: f.datatype=CS_FLOAT_TYPE
                            elif f.precision <= 9: f.datatype=CS_INT_TYPE
                        elif f.datatype in (CS_TEXT_TYPE, CS_IMAGE_TYPE):
                            # Insurance
                            if f.maxlength > 65500: f.maxlength=65500 
                        
                        buf=Buffer(f.datatype, 1, f.maxlength+1)
                        cop=Buffer(CS_INT_TYPE,1)
                        ind=Buffer(CS_SMALLINT_TYPE,1)
                        f.format=CS_FMT_UNUSED
                        if ( cm.ct_bind(si, f, buf, cop, ind) != CS_SUCCEED):
                            self._error()
                        bindings[i]=i, buf, cop, ind, f.datatype
                        dd[i]={'name': f.name, 'type': type_map[f.datatype]}

                    n=0
                    while 1:
                        if n==max_rows:
                            cm.ct_cancel(sy_occ.CS_CANCEL_CURRENT)
                            break

                        rc=cm.ct_fetch()
                        if rc==CS_ROW_FAIL:
                            self._warn("Failed a row fetch")
                            continue
                        elif rc==CS_END_DATA:
                            break
                        elif rc==CS_SUCCEED:
                            n=n+1 # increment row count
                            row=range(nv)
                            for i, buf, cop, ind, t in bindings:
                                if ind[0] < 0: row[i]=mv
                                else:
                                    if string_type(t):
                                        v=rstrip(buf[:cop[0]])
                                    elif t is sy_occ.CS_NUMERIC_TYPE:
                                        v=buf[0]
                                        x=0
                                        l=cop[0]-3
                                        m=1L
                                        while l > 0:
                                           x=x+m*ord(v[2][l])
                                           m=m*256
                                           l=l-1

                                        if ord(v[2][0]): x= -x
                                        if v[1]: x=x/(10.0**v[1])
                                        v=x
                                    else:
                                        v=buf[0]
                                        if type(v) is TupleType:
                                            v=apply(converters[t], v)
                                            
                                    row[i]=v
                            result.append(tuple(row))
                        

                    # cm.ct_bind() # undo bindings
                    
                elif rt in (CS_CMD_SUCCEED, CS_CMD_DONE): continue    
                elif rt == CS_CMD_FAIL:
                    failed=1
                    continue

                else: raise Error("Unexpected result type")

            if failed: self._error('Error processing %s' % query)

        return dd, result

    def _error(self, message=None, warn=0):
        m=[]
        if message: m.append(message)
        db=self._db
        fail=0

        rc, n = db.ct_diag(sy_occ.CS_STATUS, sy_occ.CS_SERVERMSG_TYPE)
        if n:
            for i in range(1,n+1):
                msg=db.ct_diag(sy_occ.CS_GET, sy_occ.CS_SERVERMSG_TYPE, i)[1]
                if msg.severity != sy_occ.CS_SV_INFORM: fail=1
                m.append(msg.text)
                
            db.ct_diag(sy_occ.CS_CLEAR, sy_occ.CS_SERVERMSG_TYPE)

        rc, n = db.ct_diag(sy_occ.CS_STATUS, sy_occ.CS_CLIENTMSG_TYPE)
        if n:
            for i in range(1,n+1):
                msg=db.ct_diag(sy_occ.CS_GET, sy_occ.CS_CLIENTMSG_TYPE, i)[1]
                if msg.severity != sy_occ.CS_SV_INFORM: fail=1
                m.append(msg.msgstring)
                
            db.ct_diag(sy_occ.CS_CLEAR, sy_occ.CS_CLIENTMSG_TYPE)

        m=join(m,'\n')
        if warn:
            if fail:
                LOG('ZSybaseDA', PROBLEM, m)
            return m
        else:
            raise Error(m)

    def _warn(self, message=None): return self._error(message, 1)


    # Zope transaction management support.
    _subt=_save_name=None

    def _begin(self):
        if not self._subt:
            self.query('begin transaction', simple=1)
    
    def tpc_begin(self, t, subtransaction=None):
        self._subt=subtransaction

    def tpc_finish(self, *ignored):
        if self._subt:
            _save_name='sv_%s' % self.id
            self.query('save transaction %s' % _save_name, simple=1)
            self._save_name=_save_name
        else:
            self.query('commit transaction', simple=1)
        self._registered=0

    def tpc_abort(self, *ignored):
        if self._subt and self._save_name:
            self.query('rollback transaction %s' % self._save_name, simple=1)
        else:
            self.query('rollback transaction', simple=1)
            self._save_name=None
        self._registered=0

    abort=tpc_abort

    def commit_sub(self, *ignored):
        self._subt=None
        self._save_name=None
        self.query('commit transaction', simple=1)

    def abort_sub(self, *ignored):
        self._subt=None
        self._save_name=None
        self.query('rollback transaction', simple=1)
