/*

  Copyright (c) 1998, Digital Creations, Fredericksburg, VA, USA.  
  All rights reserved.
  
  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are
  met:
  
    o Redistributions of source code must retain the above copyright
      notice, this list of conditions, and the disclaimer that follows.
  
    o Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions, and the following disclaimer in
      the documentation and/or other materials provided with the
      distribution.
  
    o Neither the name of Digital Creations nor the names of its
      contributors may be used to endorse or promote products derived
      from this software without specific prior written permission.
  
  
  THIS SOFTWARE IS PROVIDED BY DIGITAL CREATIONS AND CONTRIBUTORS *AS
  IS* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
  TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
  PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL DIGITAL
  CREATIONS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
  USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
  DAMAGE.

  If you have questions regarding this software,
  contact:
 
    Digital Creations L.C.  
    info@digicool.com
 
    (540) 371-6909

*/

				
/*  

    INCOMPLETE CALLBACK SUPPORT

    WARNING

    Callbacks are *really* hard. Don't try to anable them.

*/


static CS_RETCODE 
csmsg_callback(CS_CONTEXT *ctx, CS_CLIENTMSG *e)
{
  PyObject *context, *msg, *cb;
  CS_INT outlen;
  CS_RETCODE r;

  r=cs_config(ctx, CS_GET, CS_USERDATA,
	      (CS_VOID*)&context, (CS_INT)sizeof(PyObject*), &outlen);
   
  if (r != CS_SUCCEED || ! context || context->ob_type != &ContextType
      || ! ((Context*)context)->callbacks)
    return CS_SUCCEED;

  /* This looks backward, but that's because this method
     represents a call from Syase to Python, which is the
     reverse of the normal call direction.
   */
  Py_END_ALLOW_THREADS

  UNLESS (cb=PyDict_GetItem(((Context*)context)->callbacks, Py_None))
    {
      PyErr_Clear();
      goto bail;
    }
    
  UNLESS (msg=OBJECT(newClientMSG(e))) 
    {
      PyErr_Clear();
      goto bail;
    }

  if ((cb=PyObject_CallFunction(cb, "OO", context, msg)))
    {  
      if (PyInt_Check(cb) && (CS_INT)PyInt_AsLong(cb)==CS_FAIL) r=CS_FAIL;
      Py_DECREF(cb);
    }
  else PyErr_Clear();
  
  Py_BEGIN_ALLOW_THREADS
  return r;

bail:
  Py_BEGIN_ALLOW_THREADS
  return CS_SUCCEED;  
}

static CS_RETCODE 
clientmessage_callback(CS_CONTEXT *ctx, CS_CONNECTION *con, CS_CLIENTMSG *e)
{
  PyObject *connection, *msg, *cb;
  CS_INT outlen;
  CS_RETCODE ret;

  if (! con) return csmsg_callback(ctx, e);

  ret=ct_con_props(con, CS_GET, CS_USERDATA,
		   (CS_VOID*)&connection, (CS_INT)sizeof(PyObject*), &outlen);

  if (ret != CS_SUCCEED || ! connection || 
      connection->ob_type != &ConnectionType || 
      ! ((Connection*)connection)->callbacks)
    return CS_SUCCEED;

  /* This looks backward, but that's because this method
     represents a call from Syase to Python, which is the
     reverse of the normal call direction.
   */
  Py_END_ALLOW_THREADS

  UNLESS (cb=PyDict_GetItem(((Connection*)connection)->callbacks, Py_None))
    {
      PyErr_Clear();
      goto bail;
    }
    
  UNLESS (msg=OBJECT(newClientMSG(e)))
    {
      PyErr_Clear();
      goto bail;
    }

  if ((cb=PyObject_CallFunction(cb, "OO", connection, msg)))
    {  
      if (PyInt_Check(cb) && (CS_INT)PyInt_AsLong(cb)==CS_FAIL) ret=CS_FAIL;
      Py_DECREF(cb);
    }
  else PyErr_Clear();
  
  return ret;

bail:
  Py_BEGIN_ALLOW_THREADS
  return CS_SUCCEED;  
}


static CS_RETCODE 
ssmsg_callback(CS_CONTEXT *ctx, CS_SERVERMSG *e)
{
  PyObject *context, *msg, *cb;
  CS_INT outlen;
  CS_RETCODE ret;

  ret=cs_config(ctx, CS_GET, CS_USERDATA,
		(CS_VOID*)&context, (CS_INT)sizeof(PyObject*), &outlen);

  if (ret != CS_SUCCEED || ! context || context->ob_type != &ContextType
      || ! ((Context*)context)->callbacks)
    return CS_SUCCEED;

  /* This looks backward, but that's because this method
     represents a call from Syase to Python, which is the
     reverse of the normal call direction.
   */
  Py_END_ALLOW_THREADS

  UNLESS (cb=PyDict_GetItem(((Context*)context)->callbacks, Py_None))
    {
      PyErr_Clear();
      goto bail;
    }
    
  UNLESS (msg=OBJECT(newServerMSG(e))) 
    {
      PyErr_Clear();
      goto bail;
    }

  if ((cb=PyObject_CallFunction(cb, "OO", context, msg)))
    {  
      if (PyInt_Check(cb) && (CS_INT)PyInt_AsLong(cb)==CS_FAIL) ret=CS_FAIL;
      Py_DECREF(cb);
    }
  else PyErr_Clear();
  
  Py_BEGIN_ALLOW_THREADS
  return ret;

bail:
  Py_BEGIN_ALLOW_THREADS
  return CS_SUCCEED;  
}


static CS_RETCODE 
servermessage_callback(CS_CONTEXT *ctx, CS_CONNECTION *con, CS_SERVERMSG *e)
{
  PyObject *connection, *msg, *cb;
  CS_INT outlen;
  CS_RETCODE ret;

  if (! con) return ssmsg_callback(ctx, e);

  Py_BEGIN_ALLOW_THREADS
  ret=ct_con_props(con, CS_GET, CS_USERDATA,
		   (CS_VOID*)&connection, (CS_INT)sizeof(PyObject*), &outlen);
  Py_END_ALLOW_THREADS

  if (ret != CS_SUCCEED || ! connection 
      || connection->ob_type != &ConnectionType
      || ! ((Connection*)connection)->callbacks)
    return CS_SUCCEED;

  UNLESS (cb=PyDict_GetItem(((Connection*)connection)->callbacks, Py_None))
    {
      PyErr_Clear();
      return CS_SUCCEED;
    }
    
  UNLESS (msg=OBJECT(newServerMSG(e)))
    {
      PyErr_Clear();
      return CS_SUCCEED;
    }

  if ((cb=PyObject_CallFunction(cb, "OO", connection, msg)))
    {  
      if (PyInt_Check(cb) && (CS_INT)PyInt_AsLong(cb)==CS_FAIL) ret=CS_FAIL;
      Py_DECREF(cb);
    }
  else PyErr_Clear();
  
  return ret;
}


static int
Context_allocCallbacks(Context *self)
{
  CS_INT outlen;
  CS_RETCODE r;

  Py_BEGIN_ALLOW_THREADS
  r=cs_config(self->context, CS_SET, CS_USERDATA,
	      &self, sizeof(Context*), NULL);
  Py_END_ALLOW_THREADS

  if (r != CS_SUCCEED)
    {
      SybaseError("Couldn't set user data to for callbacks");
      return -1;
    }

  UNLESS (self->callbacks=PyDict_New()) return -1;

  return 0;
}

static CS_RETCODE 
completion_callback(CS_CONNECTION *con, CS_COMMAND *cmd, 
		    CS_INT function, CS_RETCODE status)
{
  PyObject *connection, *cb, *ccmd;
  CS_INT outlen;
  CS_RETCODE ret;
  CS_CLIENTMSG msg;

  Py_BEGIN_ALLOW_THREADS
  ret=ct_con_props(con, CS_GET, CS_USERDATA,
		   (CS_VOID*)&connection, (CS_INT)sizeof(PyObject*), &outlen);
  Py_END_ALLOW_THREADS

  if (ret != CS_SUCCEED || ! connection 
      || connection->ob_type != &ConnectionType
      || ! ((Connection*)connection)->callbacks)
    return CS_SUCCEED;

  UNLESS (cb=PyDict_GetItem(((Connection*)connection)->callbacks, Py_None))
    {
      PyErr_Clear();
      return CS_SUCCEED;
    }

  if (cmd) 
    {
      UNLESS (ccmd=OBJECT(PyCObject_FromVoidPtr(cmd, NULL)))
	{
	  PyErr_Clear();
	  return CS_SUCCEED;
	}
    }
  else ccmd=Py_None;
  
  if ((cb=PyObject_CallFunction(cb, "OOii", 
				connection, ccmd, (int)function, (int)status)))
    {  
      if (PyInt_Check(cb) && (CS_INT)PyInt_AsLong(cb)==CS_FAIL) ret=CS_FAIL;
      Py_DECREF(cb);
    }
  else PyErr_Clear();
  
  return ret;
}

static PyObject *
Context_callback(Context *self, PyObject *args)
{
  int type;
  PyObject *t, *o, *f=NULL;
  CS_VOID *buf;
  CS_RETCODE r;

  UNLESS (PyArg_ParseTuple(args,"O!|O", &PyInt_Type, &type, &f)) return NULL;

  type=PyInt_AsLong(t);

  if (! f)
    {
      if (self->callbacks)
	{
	  o=PyDict_GetItem(self->callbacks, t);
	  if (! o)
	    {
	      PyErr_Clear();
	      o=Py_None;
	    }
	}
      else o=Py_None;
      
      Py_INCREF(o);
      return o;
    }

  if (! self->callbacks && Context_allocCallbacks(self) < 0) return NULL;

  if (PyDict_SetItem(self->callbacks, Py_None, f) < 0) return NULL;

  if (f == Py_None) buf=NULL;
  else
    switch (type)
      {
      case CS_CLIENTMSG_CB: 
	buf=&clientmessage_callback;
	break;
      case CS_SERVERMSG_CB: 
	buf=&servermessage_callback;
	break;
      case CS_COMPLETION_CB:
	buf=&completion_callback;
	break;
      default:
	return SybaseError("Invalid callback");
      }

  Py_BEGIN_ALLOW_THREADS
  r=ct_callback(self->context, NULL, CS_SET, (CS_INT)type, buf);
  Py_END_ALLOW_THREADS

  return PyInt_FromLong((long));
}

static int
Connection_allocCallbacks(Connection *self)
{
  CS_INT outlen; 
  CS_RETCODE r;

  Py_BEGIN_ALLOW_THREADS
  r=ct_con_props(self->connection, CS_SET, CS_USERDATA,
		 &self, sizeof(Context*), NULL);
  Py_END_ALLOW_THREADS

  if ( != CS_SUCCEED)
    {
      SybaseError("Couldn't set user data to for callbacks");
      return -1;
    }
  
  UNLESS (self->callbacks=PyDict_New()) return -1;

  return 0;
}

static PyObject *
Connection_callback(Connection *self, PyObject *args)
{
  int type;
  PyObject *t, *f=NULL, *o;
  CS_VOID *buf;
  CS_RETCODE r;

  UNLESS (PyArg_ParseTuple(args,"O!|O", &PyInt_Type, &type, &f)) return NULL;

  type=PyInt_AsLong(t);

  if (! f)
    {
      if (self->callbacks)
	{
	  o=PyDict_GetItem(self->callbacks, t);
	  if (! o)
	    {
	      PyErr_Clear();
	      o=Py_None;
	    }
	}
      else o=Py_None;
      
      Py_INCREF(o);
      return o;
    }

  if (! self->callbacks && Connection_allocCallbacks(self) < 0) return NULL;

  if (PyDict_SetItem(self->callbacks, Py_None, f) < 0) return NULL;

  if (f == Py_None) buf=NULL;
  else
    switch (type)
      {
      case CS_CLIENTMSG_CB: 
	buf=&clientmessage_callback;
	break;
      case CS_SERVERMSG_CB: 
	buf=&servermessage_callback;
	break;
      case CS_COMPLETION_CB:
	buf=&completion_callback;
	break;
      default:
	return SybaseError("Invalid callback");
      }

  Py_BEGIN_ALLOW_THREADS
  r=ct_callback(NULL, self->connection, CS_SET, (CS_INT)type, buf);
  Py_END_ALLOW_THREADS

  return PyInt_FromLong(r(long));
}
