/*

  Copyright (c) 1998, Digital Creations, Fredericksburg, VA, USA.  
  All rights reserved.
  
  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are
  met:
  
    o Redistributions of source code must retain the above copyright
      notice, this list of conditions, and the disclaimer that follows.
  
    o Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions, and the following disclaimer in
      the documentation and/or other materials provided with the
      distribution.
  
    o Neither the name of Digital Creations nor the names of its
      contributors may be used to endorse or promote products derived
      from this software without specific prior written permission.
  
  
  THIS SOFTWARE IS PROVIDED BY DIGITAL CREATIONS AND CONTRIBUTORS *AS
  IS* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
  TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
  PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL DIGITAL
  CREATIONS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
  USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
  DAMAGE.

  If you have questions regarding this software,
  contact:
 
    Digital Creations L.C.  
    info@digicool.com
 
    (540) 371-6909

*/

static int DEFINES(PyObject *d)
{
  PyObject *v;

#define CONSTANT(N) \
  UNLESS (v=PyInt_FromLong((long)(N))) return -1; \
  if (PyDict_SetItemString(d, # N, v) < 0) return -1

  /*
    Constants for CS-Lib API
  */

/*****************************************************************************
**
** Size defines used in client/server applications.
**
*****************************************************************************/

/*
** Define the maximum size of a fully qualified table name.
*/
  CONSTANT(CS_OBJ_NAME);

/*
** The maximum number of bytes in the server message or error message
** stored in the CS_SERVERMSG and CS_CLIENTMSG structures. If the total
** message is longer than this, multiple structures will be passed to the
** application program.
*/
  CONSTANT(CS_MAX_MSG);

/*
** The maximum number of bytes in a Sybase timestamp.
*/
  CONSTANT(CS_TS_SIZE);

/*
** The maximum number of bytes in Sybase text pointers.
*/
  CONSTANT(CS_TP_SIZE);

/*
** The size of the sqlstate array in the CS_SERVERMSG and CS_CLIENTMSG
** structures. Please note that this size is 2 bytes greater than what the
** standard specifies. Users should look at only the first 6 bytes in
** the array. That last 2 bytes are reserved to insure modulo 4 byte
** structure alignment.
*/
  CONSTANT(CS_SQLSTATE_SIZE);

/*****************************************************************************
**
** Defines passed into Open Client/Server APIs.
**
*****************************************************************************/

/*
** Define all the library versions currently supported.
*/
  CONSTANT(CS_VERSION_100);
  CONSTANT(CS_VERSION_110);

/*
** Action flags used.
** 	CS_CACHE currently only used by OMI apis
*/
  CONSTANT(CS_GET);
  CONSTANT(CS_SET);
  CONSTANT(CS_CLEAR);
  CONSTANT(CS_INIT);
  CONSTANT(CS_STATUS);
  CONSTANT(CS_MSGLIMIT);
  CONSTANT(CS_SEND);
  CONSTANT(CS_SUPPORTED);
  CONSTANT(CS_CACHE);
 
/*
** Bind indicator values. These are preferred when passing data into
** Client Library or Server Library, since they add the appropriate cast.
*/
  CONSTANT(CS_GOODDATA);
  CONSTANT(CS_NULLDATA);

/*
** Define ct_debug() operations.
*/
  CONSTANT(CS_SET_FLAG);
  CONSTANT(CS_CLEAR_FLAG);
  CONSTANT(CS_SET_DBG_FILE);
  CONSTANT(CS_SET_PROTOCOL_FILE);

/*
** Define ct_debug() types of trace information.
*/
  CONSTANT(CS_DBG_ALL);
  CONSTANT(CS_DBG_ASYNC);
  CONSTANT(CS_DBG_ERROR);
  CONSTANT(CS_DBG_MEM);
  CONSTANT(CS_DBG_PROTOCOL);
  CONSTANT(CS_DBG_PROTOCOL_STATES);
  CONSTANT(CS_DBG_API_STATES);
  CONSTANT(CS_DBG_DIAG);
  CONSTANT(CS_DBG_NETWORK);
  CONSTANT(CS_DBG_API_LOGCALL);

/*
** Cancel types.
*/
  CONSTANT(CS_CANCEL_CURRENT);
  CONSTANT(CS_CANCEL_ALL);
  CONSTANT(CS_CANCEL_ATTN);
  CONSTANT(CS_CANCEL_ABORT_NOTIF);

/*
** Cursor fetch options. Currently these are not supported within Open
** Client and Open Server.
*/
  CONSTANT(CS_FIRST);
  CONSTANT(CS_NEXT);
  CONSTANT(CS_PREV);
  CONSTANT(CS_LAST);
  CONSTANT(CS_ABSOLUTE);
  CONSTANT(CS_RELATIVE);

/*
** Op codes used in cs_calc().
*/
  CONSTANT(CS_ADD);
  CONSTANT(CS_SUB);
  CONSTANT(CS_MULT);
  CONSTANT(CS_DIV);
  CONSTANT(CS_ZERO);

/*
** The cs_dt_info() types.
*/
  CONSTANT(CS_MONTH);
  CONSTANT(CS_SHORTMONTH);
  CONSTANT(CS_DAYNAME);
  CONSTANT(CS_DATEORDER);
  CONSTANT(CS_12HOUR);
  CONSTANT(CS_DT_CONVFMT);

/*
** The cs_strcmp() options.
*/
  CONSTANT(CS_COMPARE);
  CONSTANT(CS_SORT);

/*
** Callback types.
*/
  CONSTANT(CS_COMPLETION_CB);
  CONSTANT(CS_SERVERMSG_CB);
  CONSTANT(CS_CLIENTMSG_CB);
  CONSTANT(CS_NOTIF_CB);
  CONSTANT(CS_ENCRYPT_CB);
  CONSTANT(CS_CHALLENGE_CB);
  CONSTANT(CS_DS_LOOKUP_CB);
  CONSTANT(CS_SECSESSION_CB);

/*
** To install a signal callback, the type needs to calculated as an
** offset of the operating-system-specific signal number and the
** following define. For example, to install a callback handler for a
** SIGALRM signal, pass (CS_SIGNAL_CB + SIGALRM) to the ct_callback()
** routine.
*/
  CONSTANT(CS_SIGNAL_CB);

/*
** Exit and close flags.
*/
  CONSTANT(CS_FORCE_EXIT);
  CONSTANT(CS_FORCE_CLOSE);

/*
** ct_diag() and cs_diag() type flags.
*/
  CONSTANT(CS_CLIENTMSG_TYPE);
  CONSTANT(CS_SERVERMSG_TYPE);
  CONSTANT(CS_ALLMSG_TYPE);
  CONSTANT(SQLCA_TYPE);
  CONSTANT(SQLCODE_TYPE);
  CONSTANT(SQLSTATE_TYPE);

/*
** Compute info types.
*/
  CONSTANT(CS_COMP_OP);
  CONSTANT(CS_COMP_ID);
  CONSTANT(CS_COMP_COLID);
  CONSTANT(CS_COMP_BYLIST);
  CONSTANT(CS_BYLIST_LEN);

/*
** Compute info operators.
*/
  CONSTANT(CS_OP_SUM);
  CONSTANT(CS_OP_AVG);
  CONSTANT(CS_OP_COUNT);
  CONSTANT(CS_OP_MIN);
  CONSTANT(CS_OP_MAX);

/*
** Browse types.
*/
  CONSTANT(CS_ISBROWSE);
  CONSTANT(CS_TABNUM);
  CONSTANT(CS_TABNAME);

/*
** Result types from ct_results().
*/
  CONSTANT(CS_ROW_RESULT);
  CONSTANT(CS_CURSOR_RESULT);
  CONSTANT(CS_PARAM_RESULT);
  CONSTANT(CS_STATUS_RESULT);
  CONSTANT(CS_MSG_RESULT);
  CONSTANT(CS_COMPUTE_RESULT);
  CONSTANT(CS_CMD_DONE);
  CONSTANT(CS_CMD_SUCCEED);
  CONSTANT(CS_CMD_FAIL);
  CONSTANT(CS_ROWFMT_RESULT);
  CONSTANT(CS_COMPUTEFMT_RESULT);
  CONSTANT(CS_DESCRIBE_RESULT);

/*
** Flags for getting result info using ct_res_info().
*/
  CONSTANT(CS_ROW_COUNT);
  CONSTANT(CS_CMD_NUMBER);
  CONSTANT(CS_NUM_COMPUTES);
  CONSTANT(CS_NUMDATA);
  CONSTANT(CS_ORDERBY_COLS);
  CONSTANT(CS_NUMORDERCOLS);
  CONSTANT(CS_MSGTYPE);
  CONSTANT(CS_BROWSE_INFO);
  CONSTANT(CS_TRANS_STATE);

/*
** Possible values for CS_TRANS_STATE.
*/
  CONSTANT(CS_TRAN_UNDEFINED);
  CONSTANT(CS_TRAN_IN_PROGRESS);
  CONSTANT(CS_TRAN_COMPLETED);
  CONSTANT(CS_TRAN_FAIL);
  CONSTANT(CS_TRAN_STMT_FAIL);

/*
** Define the invalid count that the application gets when
** ct_res_info() is called at the wrong time.
*/
  CONSTANT(CS_NO_COUNT);

/*****************************************************************************
**
** Commands in Open Client/Server APIs.
**
*****************************************************************************/

/*
** ct_command() command types.
*/
  CONSTANT(CS_LANG_CMD);
  CONSTANT(CS_RPC_CMD);
  CONSTANT(CS_MSG_CMD);
  CONSTANT(CS_SEND_DATA_CMD);
  CONSTANT(CS_PACKAGE_CMD);
  CONSTANT(CS_SEND_BULK_CMD);

/*
** ct_cursor() command types.
*/
  CONSTANT(CS_CURSOR_DECLARE);
  CONSTANT(CS_CURSOR_OPEN);
  CONSTANT(CS_CURSOR_ROWS);
  CONSTANT(CS_CURSOR_UPDATE);
  CONSTANT(CS_CURSOR_DELETE);
  CONSTANT(CS_CURSOR_CLOSE);
  CONSTANT(CS_CURSOR_DEALLOC);
  CONSTANT(CS_CURSOR_OPTION);

/*
** Open Server-specific cursor command types.
*/
  CONSTANT(CS_CURSOR_FETCH);
  CONSTANT(CS_CURSOR_INFO);

/*
** ct_dyndesc() command types.
*/
  CONSTANT(CS_ALLOC);
  CONSTANT(CS_DEALLOC);
  CONSTANT(CS_USE_DESC);
  CONSTANT(CS_GETCNT);
  CONSTANT(CS_SETCNT);
  CONSTANT(CS_GETATTR);
  CONSTANT(CS_SETATTR);

/*
** ct_dynamic() command types.
*/
  CONSTANT(CS_PREPARE);
  CONSTANT(CS_EXECUTE);
  CONSTANT(CS_EXEC_IMMEDIATE);
  CONSTANT(CS_DESCRIBE_INPUT);
  CONSTANT(CS_DESCRIBE_OUTPUT);
  CONSTANT(CS_DYN_CURSOR_DECLARE);


/*
** ct_dynsqlda() arguments and actions
*/
  CONSTANT(CS_SQLDA_SYBASE);
  CONSTANT(CS_GET_IN);
  CONSTANT(CS_GET_OUT);
  CONSTANT(CS_SQLDA_BIND);
  CONSTANT(CS_SQLDA_PARAM);

/*
** Open Server-specific dynamic command types.
*/
  CONSTANT(CS_PROCNAME);
  CONSTANT(CS_ACK);


/*
** ct_ds_objinfo() objinfo types.
*/
  CONSTANT(CS_OBJ_CLASSOID);
  CONSTANT(CS_OBJ_DNAME);
  CONSTANT(CS_OBJ_NUMATTR);
  CONSTANT(CS_OBJ_ATRRIBUTE);

/*
** Command options
*/
  CONSTANT(CS_RECOMPILE);
  CONSTANT(CS_NO_RECOMPILE);
  CONSTANT(CS_BULK_INIT);
  CONSTANT(CS_BULK_CONT);
  CONSTANT(CS_BULK_DATA);
  CONSTANT(CS_COLUMN_DATA);

/*
** Cursor options.
*/
  CONSTANT(CS_FOR_UPDATE);
  CONSTANT(CS_READ_ONLY);
  CONSTANT(CS_DYNAMIC);
  CONSTANT(CS_RESTORE_OPEN);
  CONSTANT(CS_MORE);
  CONSTANT(CS_END);

/*
** Sybase-defined message ids for CS_MSG_CMDs.
*/
  CONSTANT(CS_MSG_GETLABELS);
  CONSTANT(CS_MSG_LABELS);
  CONSTANT(CS_MSG_TABLENAME);
  CONSTANT(CS_PARSE_TREE);

/*
** Minimum and maximum user-defined message id for CS_MSG_CMDs.
*/
  CONSTANT(CS_USER_MSGID);
  CONSTANT(CS_USER_MAX_MSGID);

/*
** Defines for sp_regwatch registered procedure
*/
  CONSTANT(CS_NOTIFY_ONCE);
  CONSTANT(CS_NOTIFY_ALWAYS);
  CONSTANT(CS_NOTIFY_WAIT);
  CONSTANT(CS_NOTIFY_NOWAIT);


/*****************************************************************************
**
** Open Client properties.
**
*****************************************************************************/

/*
** Properties that are used in *_props() functions.
*/
  CONSTANT(CS_USERNAME);
  CONSTANT(CS_PASSWORD);
  CONSTANT(CS_APPNAME);
  CONSTANT(CS_HOSTNAME);
  CONSTANT(CS_LOGIN_STATUS);
  CONSTANT(CS_TDS_VERSION);
  CONSTANT(CS_CHARSETCNV);
  CONSTANT(CS_PACKETSIZE);
  CONSTANT(CS_USERDATA);
  CONSTANT(CS_COMMBLOCK);
  CONSTANT(CS_NETIO);
  CONSTANT(CS_NOINTERRUPT);
  CONSTANT(CS_TEXTLIMIT);
  CONSTANT(CS_HIDDEN_KEYS);
  CONSTANT(CS_VERSION);
  CONSTANT(CS_IFILE);
  CONSTANT(CS_LOGIN_TIMEOUT);
  CONSTANT(CS_TIMEOUT);
  CONSTANT(CS_MAX_CONNECT);
  CONSTANT(CS_MESSAGE_CB);
  CONSTANT(CS_EXPOSE_FMTS);
  CONSTANT(CS_EXTRA_INF);
  CONSTANT(CS_TRANSACTION_NAME);
  CONSTANT(CS_ANSI_BINDS);
  CONSTANT(CS_BULK_LOGIN);
  CONSTANT(CS_LOC_PROP);
  CONSTANT(CS_CUR_STATUS);
  CONSTANT(CS_CUR_ID);
  CONSTANT(CS_CUR_NAME);
  CONSTANT(CS_CUR_ROWCOUNT);
  CONSTANT(CS_PARENT_HANDLE);
  CONSTANT(CS_EED_CMD);
  CONSTANT(CS_DIAG_TIMEOUT);
  CONSTANT(CS_DISABLE_POLL);
  CONSTANT(CS_NOTIF_CMD);
  CONSTANT(CS_SEC_ENCRYPTION);
  CONSTANT(CS_SEC_CHALLENGE);
  CONSTANT(CS_SEC_NEGOTIATE);
  CONSTANT(CS_MEM_POOL);
  CONSTANT(CS_USER_ALLOC);
  CONSTANT(CS_USER_FREE);
  CONSTANT(CS_ENDPOINT);
  CONSTANT(CS_NO_TRUNCATE);
  CONSTANT(CS_CON_STATUS);
  CONSTANT(CS_VER_STRING);
  CONSTANT(CS_ASYNC_NOTIFS);
  CONSTANT(CS_SERVERNAME);
  CONSTANT(CS_THREAD_RESOURCE);
  CONSTANT(CS_NOAPI_CHK);
  CONSTANT(CS_SEC_APPDEFINED);
  CONSTANT(CS_NOCHARSETCNV_REQD);
  CONSTANT(CS_STICKY_BINDS);
  CONSTANT(CS_HAVE_CMD);
  CONSTANT(CS_HAVE_BINDS);
  CONSTANT(CS_HAVE_CUROPEN);
  CONSTANT(CS_EXTERNAL_CONFIG);
  CONSTANT(CS_CONFIG_FILE);
  CONSTANT(CS_CONFIG_BY_SERVERNAME);
 
/*
** Directory Service connection properties
*/
  CONSTANT(CS_DS_CHAIN);
  CONSTANT(CS_DS_EXPANDALIAS);
  CONSTANT(CS_DS_COPY);
  CONSTANT(CS_DS_LOCALSCOPE);
  CONSTANT(CS_DS_PREFERCHAIN);
  CONSTANT(CS_DS_SCOPE);
  CONSTANT(CS_DS_SIZELIMIT);
  CONSTANT(CS_DS_TIMELIMIT);
  CONSTANT(CS_DS_PRINCIPAL);
  CONSTANT(CS_DS_REFERRAL);
  CONSTANT(CS_DS_SEARCH);
  CONSTANT(CS_DS_DITBASE);
  CONSTANT(CS_DS_FAILOVER);
  CONSTANT(CS_NET_TRANADDR);
  CONSTANT(CS_DS_PROVIDER);
  CONSTANT(CS_RETRY_COUNT);
  CONSTANT(CS_LOOP_DELAY);

/*
** Properties for Security services support 
*/
  CONSTANT(CS_SEC_NETWORKAUTH);
  CONSTANT(CS_SEC_DELEGATION);
  CONSTANT(CS_SEC_MUTUALAUTH);
  CONSTANT(CS_SEC_INTEGRITY);
  CONSTANT(CS_SEC_CONFIDENTIALITY);
  CONSTANT(CS_SEC_CREDTIMEOUT);
  CONSTANT(CS_SEC_SESSTIMEOUT);
  CONSTANT(CS_SEC_DETECTREPLAY);
  CONSTANT(CS_SEC_DETECTSEQ);
  CONSTANT(CS_SEC_DATAORIGIN);
  CONSTANT(CS_SEC_MECHANISM);
  CONSTANT(CS_SEC_CREDENTIALS);
  CONSTANT(CS_SEC_CHANBIND);
  CONSTANT(CS_SEC_SERVERPRINCIPAL);
  CONSTANT(CS_SEC_KEYTAB);

/*
** More properties
*/
  CONSTANT(CS_ABORTCHK_INTERVAL);
  CONSTANT(CS_LOGIN_TYPE);
  CONSTANT(CS_CON_KEEPALIVE);
  CONSTANT(CS_CON_TCP_NODELAY);

/*
** CS_DS_SCOPE Values
*/
  CONSTANT(CS_SCOPE_COUNTRY);
  CONSTANT(CS_SCOPE_DMD);
  CONSTANT(CS_SCOPE_WORLD);

/*
** CS_DS_SEARCH Values
*/
  CONSTANT(CS_SEARCH_OBJECT);
  CONSTANT(CS_SEARCH_ONE_LEVEL);
  CONSTANT(CS_SEARCH_SUBTREE);
 
/*
** Possible values for the CS_NETIO property.
*/
  CONSTANT(CS_SYNC_IO);
  CONSTANT(CS_ASYNC_IO);
  CONSTANT(CS_DEFER_IO);

/*
** Possible bit values for the CS_LOGIN_STATUS property.
*/
  CONSTANT(CS_CONSTAT_CONNECTED);
  CONSTANT(CS_CONSTAT_DEAD);


/*
** Possible bit values for the CS_CUR_STATUS property.
*/
  CONSTANT(CS_CURSTAT_NONE);
  CONSTANT(CS_CURSTAT_DECLARED);
  CONSTANT(CS_CURSTAT_OPEN);
  CONSTANT(CS_CURSTAT_CLOSED);
  CONSTANT(CS_CURSTAT_RDONLY);
  CONSTANT(CS_CURSTAT_UPDATABLE);
  CONSTANT(CS_CURSTAT_ROWCOUNT);
  CONSTANT(CS_CURSTAT_DEALLOC);

/*
** Possible values for the CS_TDS_VERSION property.
*/
  CONSTANT(CS_TDS_40);
  CONSTANT(CS_TDS_42);
  CONSTANT(CS_TDS_46);
  CONSTANT(CS_TDS_495);
  CONSTANT(CS_TDS_50);


/*****************************************************************************
**
** Open Client/Server options.
**
*****************************************************************************/

/*
** The following is the list of all valid options:
*/
  CONSTANT(CS_OPT_DATEFIRST);
  CONSTANT(CS_OPT_TEXTSIZE);
  CONSTANT(CS_OPT_STATS_TIME);
  CONSTANT(CS_OPT_STATS_IO);
  CONSTANT(CS_OPT_ROWCOUNT);
  CONSTANT(CS_OPT_NATLANG);
  CONSTANT(CS_OPT_DATEFORMAT);
  CONSTANT(CS_OPT_ISOLATION);

  CONSTANT(CS_OPT_AUTHON);
  CONSTANT(CS_OPT_CHARSET);
  CONSTANT(CS_OPT_SHOWPLAN);
 
  CONSTANT(CS_OPT_NOEXEC);
  CONSTANT(CS_OPT_ARITHIGNORE);

  CONSTANT(CS_OPT_TRUNCIGNORE);
  CONSTANT(CS_OPT_ARITHABORT);

  CONSTANT(CS_OPT_PARSEONLY);

  CONSTANT(CS_OPT_GETDATA);
  CONSTANT(CS_OPT_NOCOUNT);
  CONSTANT(CS_OPT_FORCEPLAN);

  CONSTANT(CS_OPT_FORMATONLY);
  CONSTANT(CS_OPT_CHAINXACTS);
  CONSTANT(CS_OPT_CURCLOSEONXACT);
  CONSTANT(CS_OPT_FIPSFLAG);
  CONSTANT(CS_OPT_RESTREES);
  CONSTANT(CS_OPT_IDENTITYON);
  CONSTANT(CS_OPT_CURREAD);

  CONSTANT(CS_OPT_CURWRITE);

  CONSTANT(CS_OPT_IDENTITYOFF);
  CONSTANT(CS_OPT_AUTHOFF);
  CONSTANT(CS_OPT_ANSINULL);
  CONSTANT(CS_OPT_QUOTED_IDENT);
  CONSTANT(CS_OPT_ANSIPERM);
  CONSTANT(CS_OPT_STR_RTRUNC);
 
  CONSTANT(CS_MIN_OPTION);
  CONSTANT(CS_MAX_OPTION);

/*
** The supported options are summarized below with their defined values
** for `ArgSize' and `OptionArg'. ArgSize specifies the domain of valid
** values that are allowed.
** 
** Option			ArgSize		OptionArg
** ---------------		---------	---------
** CS_OPT_DATEFIRST 		1 byte		Defines below
** CS_OPT_TEXTSIZE 		4 bytes		Size in bytes 
** CS_OPT_ROWCOUNT  		4 bytes		Number of rows 
** CS_OPT_NATLANG 		OptionArg Len	National Lang (string)
** CS_OPT_DATEFORMAT 		1 byte		Defines below
** CS_OPT_ISOLATION 		1 byte		Defines below		
** CS_OPT_AUTHON 		OptionArg Len	Table Name (string)
** CS_OPT_CHARSET 		OptionArg Len	Character set (string)
** CS_OPT_IDENTITYON 		OptionArg Len	Table Name (string)
** CS_OPT_CURREAD 		OptionArg Len	Read Label(string)
** CS_OPT_CURWRITE 		OptionArg Len	Write Label(string)
** CS_OPT_IDENTITYOFF 		OptionArg Len	Table Name (string)
** CS_OPT_AUTHOFF 		OptionArg Len	Table Name (string)
** (All remaining options)	1 byte		Boolean value
** 
** All string values must be sent in 7 bit ASCII.
**
*/ 


/* CS_OPT_DATEFIRST */
  CONSTANT(CS_OPT_MONDAY);
  CONSTANT(CS_OPT_TUESDAY);
  CONSTANT(CS_OPT_WEDNESDAY);
  CONSTANT(CS_OPT_THURSDAY);
  CONSTANT(CS_OPT_FRIDAY);
  CONSTANT(CS_OPT_SATURDAY);
  CONSTANT(CS_OPT_SUNDAY);

/* CS_OPT_DATEFORMAT */
  CONSTANT(CS_OPT_FMTMDY);
  CONSTANT(CS_OPT_FMTDMY);
  CONSTANT(CS_OPT_FMTYMD);
  CONSTANT(CS_OPT_FMTYDM);
  CONSTANT(CS_OPT_FMTMYD);
  CONSTANT(CS_OPT_FMTDYM);

/* CS_OPT_ISOLATION */
  CONSTANT(CS_OPT_LEVEL0);
  CONSTANT(CS_OPT_LEVEL1);
  CONSTANT(CS_OPT_LEVEL3);

/*****************************************************************************
**
** Open Client/Server capabilities.
**
*****************************************************************************/

/*
** Capability types.
*/
  CONSTANT(CS_CAP_REQUEST);
  CONSTANT(CS_CAP_RESPONSE);

/*
** Special capability value to set/get all capability values at once.
*/
  CONSTANT(CS_ALL_CAPS);

/*
** Capability request values.
*/
  CONSTANT(CS_REQ_LANG);
  CONSTANT(CS_REQ_RPC);
  CONSTANT(CS_REQ_NOTIF);
  CONSTANT(CS_REQ_MSTMT);
  CONSTANT(CS_REQ_BCP);
  CONSTANT(CS_REQ_CURSOR);
  CONSTANT(CS_REQ_DYN);
  CONSTANT(CS_REQ_MSG);
  CONSTANT(CS_REQ_PARAM);
  CONSTANT(CS_DATA_INT1);
  CONSTANT(CS_DATA_INT2);
  CONSTANT(CS_DATA_INT4);
  CONSTANT(CS_DATA_BIT);
  CONSTANT(CS_DATA_CHAR);
  CONSTANT(CS_DATA_VCHAR);
  CONSTANT(CS_DATA_BIN);
  CONSTANT(CS_DATA_VBIN);
  CONSTANT(CS_DATA_MNY8);
  CONSTANT(CS_DATA_MNY4);
  CONSTANT(CS_DATA_DATE8);
  CONSTANT(CS_DATA_DATE4);
  CONSTANT(CS_DATA_FLT4);
  CONSTANT(CS_DATA_FLT8);
  CONSTANT(CS_DATA_NUM);
  CONSTANT(CS_DATA_TEXT);
  CONSTANT(CS_DATA_IMAGE);
  CONSTANT(CS_DATA_DEC);
  CONSTANT(CS_DATA_LCHAR);
  CONSTANT(CS_DATA_LBIN);
  CONSTANT(CS_DATA_INTN);
  CONSTANT(CS_DATA_DATETIMEN);
  CONSTANT(CS_DATA_MONEYN);
  CONSTANT(CS_CSR_PREV);
  CONSTANT(CS_CSR_FIRST);
  CONSTANT(CS_CSR_LAST);
  CONSTANT(CS_CSR_ABS);
  CONSTANT(CS_CSR_REL);
  CONSTANT(CS_CSR_MULTI);
  CONSTANT(CS_CON_OOB);
  CONSTANT(CS_CON_INBAND);
  CONSTANT(CS_CON_LOGICAL);
  CONSTANT(CS_PROTO_TEXT);
  CONSTANT(CS_PROTO_BULK);
  CONSTANT(CS_REQ_URGNOTIF);
  CONSTANT(CS_DATA_SENSITIVITY);
  CONSTANT(CS_DATA_BOUNDARY);
  CONSTANT(CS_PROTO_DYNAMIC);
  CONSTANT(CS_PROTO_DYNPROC);
  CONSTANT(CS_DATA_FLTN);
  CONSTANT(CS_DATA_BITN);
  CONSTANT(CS_OPTION_GET);
  CONSTANT(CS_DATA_INT8);
  CONSTANT(CS_DATA_VOID);
  CONSTANT(CS_DOL_BULK);

/*
** Minimum and maximum request capability values.
*/
  CONSTANT(CS_MIN_REQ_CAP);
  CONSTANT(CS_MAX_REQ_CAP);

/*
** Capability response values.
*/
  CONSTANT(CS_RES_NOMSG);
  CONSTANT(CS_RES_NOEED);
  CONSTANT(CS_RES_NOPARAM);
  CONSTANT(CS_DATA_NOINT1);
  CONSTANT(CS_DATA_NOINT2);
  CONSTANT(CS_DATA_NOINT4);
  CONSTANT(CS_DATA_NOBIT);
  CONSTANT(CS_DATA_NOCHAR);
  CONSTANT(CS_DATA_NOVCHAR);
  CONSTANT(CS_DATA_NOBIN);
  CONSTANT(CS_DATA_NOVBIN);
  CONSTANT(CS_DATA_NOMNY8);
  CONSTANT(CS_DATA_NOMNY4);
  CONSTANT(CS_DATA_NODATE8);
  CONSTANT(CS_DATA_NODATE4);
  CONSTANT(CS_DATA_NOFLT4);
  CONSTANT(CS_DATA_NOFLT8);
  CONSTANT(CS_DATA_NONUM);
  CONSTANT(CS_DATA_NOTEXT);
  CONSTANT(CS_DATA_NOIMAGE);
  CONSTANT(CS_DATA_NODEC);
  CONSTANT(CS_DATA_NOLCHAR);
  CONSTANT(CS_DATA_NOLBIN);
  CONSTANT(CS_DATA_NOINTN);
  CONSTANT(CS_DATA_NODATETIMEN);
  CONSTANT(CS_DATA_NOMONEYN);
  CONSTANT(CS_CON_NOOOB);
  CONSTANT(CS_CON_NOINBAND);
  CONSTANT(CS_PROTO_NOTEXT);
  CONSTANT(CS_PROTO_NOBULK);
  CONSTANT(CS_DATA_NOSENSITIVITY);
  CONSTANT(CS_DATA_NOBOUNDARY);
  CONSTANT(CS_RES_NOTDSDEBUG);
  CONSTANT(CS_RES_NOSTRIPBLANKS);
  CONSTANT(CS_DATA_NOINT8);

/*
** Minimum and maximum response capability values.
*/
  CONSTANT(CS_MIN_RES_CAP);
  CONSTANT(CS_MAX_RES_CAP);

/*
** Minimum and maximum of all capabilities defined above.
*/
  CONSTANT(CS_MIN_CAPVALUE);
  CONSTANT(CS_MAX_CAPVALUE);

/*
** Size of area to store capabilities. The array len must be greater than
S_CAP_ARRAYLEN
** additional capabilities to be added.
*/
  CONSTANT(CS_CAP_ARRAYLEN);

/*
** Maximum OID length (bytes)
*/
  CONSTANT(CS_MAX_OIDLEN);

/*****************************************************************************
**
** Defines used in Open Client/Server structures.
**
*****************************************************************************/

/*
** Define I/O types in the CS_IODESC structure.
*/
  CONSTANT(CS_IODATA);

/*
** Define status values for the status field of the CS_SERVERMSG and
** CS_CLIENTMSG structures.
*/
  CONSTANT(CS_HASEED);
  CONSTANT(CS_FIRST_CHUNK);
  CONSTANT(CS_LAST_CHUNK);

  /*
    Constants for CT-Lib API
  */
  CONSTANT(CT_BIND);
  CONSTANT(CT_BR_COLUMN);
  CONSTANT(CT_BR_TABLE);
  CONSTANT(CT_CALLBACK);
  CONSTANT(CT_CANCEL);
  CONSTANT(CT_CAPABILITY);
  CONSTANT(CT_CLOSE);
  CONSTANT(CT_CMD_ALLOC);
  CONSTANT(CT_CMD_DROP);
  CONSTANT(CT_CMD_PROPS);
  CONSTANT(CT_COMMAND);
  CONSTANT(CT_COMPUTE_INFO);
  CONSTANT(CT_CON_ALLOC);
  CONSTANT(CT_CON_DROP);
  CONSTANT(CT_CON_PROPS);
  CONSTANT(CT_CON_XFER);
  CONSTANT(CT_CONFIG);
  CONSTANT(CT_CONNECT);
  CONSTANT(CT_CURSOR);
  CONSTANT(CT_DATA_INFO);
  CONSTANT(CT_DEBUG);
  CONSTANT(CT_DESCRIBE);
  CONSTANT(CT_DIAG);
  CONSTANT(CT_DYNAMIC);
  CONSTANT(CT_DYNDESC);
  CONSTANT(CT_EXIT);
  CONSTANT(CT_FETCH);
  CONSTANT(CT_GET_DATA);
  CONSTANT(CT_GETFORMAT);
  CONSTANT(CT_GETLOGINFO);
  CONSTANT(CT_INIT);
  CONSTANT(CT_KEYDATA);
  CONSTANT(CT_OPTIONS);
  CONSTANT(CT_PARAM);
  CONSTANT(CT_POLL);
  CONSTANT(CT_RECVPASSTHRU);
  CONSTANT(CT_REMOTE_PWD);
  CONSTANT(CT_RES_INFO);
  CONSTANT(CT_RESULTS);
  CONSTANT(CT_SEND);
  CONSTANT(CT_SEND_DATA);
  CONSTANT(CT_SENDPASSTHRU);
  CONSTANT(CT_SETLOGINFO);
  CONSTANT(CT_WAKEUP);
  CONSTANT(CT_LABELS);
  CONSTANT(CT_DS_LOOKUP);
  CONSTANT(CT_DS_DROP);
  CONSTANT(CT_DS_OBJINFO);
  CONSTANT(CT_SETPARAM);
  CONSTANT(CT_DYNSQLDA);
  CONSTANT(CT_NOTIFICATION);
  CONSTANT(CT_USER_FUNC);


  /* cstypes.h ===========================================================*/



/*****************************************************************************
**
** Return code defines used in client/server applications.
**
*****************************************************************************/

/*
** See the csconfig.h header file for the definition of CS_RETCODE.
*/

/*
** Define the core return codes.
*/
  CONSTANT(CS_SUCCEED);
  CONSTANT(CS_FAIL);
  CONSTANT(CS_MEM_ERROR);
  CONSTANT(CS_PENDING);
  CONSTANT(CS_QUIET);
  CONSTANT(CS_BUSY);
  CONSTANT(CS_INTERRUPT);
  CONSTANT(CS_BLK_HAS_TEXT);
  CONSTANT(CS_CONTINUE);
  CONSTANT(CS_FATAL);

/*
** Define error offsets. All other CS_RETCODE error codes should be
** based off of these values to avoid define conflicts.
*/
  CONSTANT(CS_CONV_ERR);
  CONSTANT(CS_EXTERNAL_ERR);
  CONSTANT(CS_INTERNAL_ERR);

/* common library errors */
  CONSTANT(CS_COMN_ERR);

/*
** Return code for a routine which was cancelled via ct_cancel().
*/
  CONSTANT(CS_CANCELED);

/*
** Special returns for ct_fetch().
*/
  CONSTANT(CS_ROW_FAIL);
  CONSTANT(CS_END_DATA);

/*
** Special return for ct_results().
*/
  CONSTANT(CS_END_RESULTS);

/*
** Special return for ct_get_data().
*/
  CONSTANT(CS_END_ITEM);

/*
** Special return for ct_diag().
*/
  CONSTANT(CS_NOMSG);

/*
** Special return for ct_poll().
*/
  CONSTANT(CS_TIMED_OUT);

/*
** Special returns for passthru().
*/
  CONSTANT(CS_PASSTHRU_EOM);
  CONSTANT(CS_PASSTHRU_MORE);

/*
** Special return for ct_cancel().
*/
  CONSTANT(CS_TRYING);

/*
** Errors caused by external events (i.e. bad params supplied by user).
*/
  CONSTANT(CS_EBADPARAM);
  CONSTANT(CS_EBADLEN);
  CONSTANT(CS_ENOCNVRT);

/*
** Return codes for conversion errors. These should be used in any user-
** defined conversion routines that are install via cs_set_convert().
*/
  CONSTANT(CS_EOVERFLOW);
  CONSTANT(CS_EUNDERFLOW);
  CONSTANT(CS_EPRECISION);
  CONSTANT(CS_ESCALE);
  CONSTANT(CS_ESYNTAX);
  CONSTANT(CS_EFORMAT);
  CONSTANT(CS_EDOMAIN);
  CONSTANT(CS_EDIVZERO);
  CONSTANT(CS_ERESOURCE);
  CONSTANT(CS_ENULLNOIND);
  CONSTANT(CS_ETRUNCNOIND);
  CONSTANT(CS_ENOBIND);
  CONSTANT(CS_TRUNCATED);
  CONSTANT(CS_ESTYLE);
  CONSTANT(CS_EBADXLT);
  CONSTANT(CS_ENOXLT);
  CONSTANT(CS_USEREP);

/*
** Error Severities. 
*/
  CONSTANT(CS_SV_INFORM);
  CONSTANT(CS_SV_API_FAIL);
  CONSTANT(CS_SV_RETRY_FAIL);
  CONSTANT(CS_SV_RESOURCE_FAIL);
  CONSTANT(CS_SV_CONFIG_FAIL);
  CONSTANT(CS_SV_COMM_FAIL);
  CONSTANT(CS_SV_INTERNAL_FAIL);
  CONSTANT(CS_SV_FATAL);


/*****************************************************************************
**
** Defines used in client/server applications.
**
*****************************************************************************/

/*
** The maximum localization name allowed. A four byte value is added (instead
** of 1 byte) to provide space for null termination and remain on a modulo 4
** byte boundary.
*/
  CONSTANT(CS_MAX_LOCALE);

/*
** The maximum column name allowed.
*/
  CONSTANT(CS_MAX_NAME);

/*
** The maximum password length allowed.
*/
  CONSTANT(CS_MAX_PASS);

/*
** The maximum number of characters in arrays. Please note that this define
** does not correspond to any server definition of lengths (particularly
** the length of the character data type, which is 255 bytes for the Sybase
** server).
*/
  CONSTANT(CS_MAX_CHAR);

/*
** Maximum string in Directory Services
*/
  CONSTANT(CS_MAX_DS_STRING);

/*
** Maximum data length of numeric/decimal datatypes.
*/
  CONSTANT(CS_MAX_NUMLEN);

/*
** To be true or not true.
*/
  CONSTANT(CS_TRUE);
  CONSTANT(CS_FALSE);

/*
** Define basic default types.
*/
  CONSTANT(CS_NULLTERM);
  CONSTANT(CS_WILDCARD);
  CONSTANT(CS_NO_LIMIT);
  CONSTANT(CS_UNUSED);

/*
** timeout values
**
**	CS_NO_LIMIT		Resource will never expire.
**	CS_UNEXPIRED		Resource did not expire.
*/
  CONSTANT(CS_UNEXPIRED);

/*
** Enumerate what datatypes we support.
*/
  CONSTANT(CS_ILLEGAL_TYPE);
  CONSTANT(CS_CHAR_TYPE);
  CONSTANT(CS_BINARY_TYPE);
  CONSTANT(CS_LONGCHAR_TYPE);
  CONSTANT(CS_LONGBINARY_TYPE);
  CONSTANT(CS_TEXT_TYPE);
  CONSTANT(CS_IMAGE_TYPE);
  CONSTANT(CS_TINYINT_TYPE);
  CONSTANT(CS_SMALLINT_TYPE);
  CONSTANT(CS_INT_TYPE);
  CONSTANT(CS_REAL_TYPE);
  CONSTANT(CS_FLOAT_TYPE);
  CONSTANT(CS_BIT_TYPE);
  CONSTANT(CS_DATETIME_TYPE);
  CONSTANT(CS_DATETIME4_TYPE);
  CONSTANT(CS_MONEY_TYPE);
  CONSTANT(CS_MONEY4_TYPE);
  CONSTANT(CS_NUMERIC_TYPE);
  CONSTANT(CS_DECIMAL_TYPE);
  CONSTANT(CS_VARCHAR_TYPE);
  CONSTANT(CS_VARBINARY_TYPE);
  CONSTANT(CS_LONG_TYPE);
  CONSTANT(CS_SENSITIVITY_TYPE);
  CONSTANT(CS_BOUNDARY_TYPE);
  CONSTANT(CS_VOID_TYPE);
  CONSTANT(CS_USHORT_TYPE);

/*
** Define the minimum and maximum datatype values.
*/
  CONSTANT(CS_MIN_SYBTYPE);
  CONSTANT(CS_MAX_SYBTYPE);

/*
** Define the number of datatypes that are supported by Sybase.
*/
  CONSTANT(CS_MAXSYB_TYPE);

/*
** The minumum user-defined data type.
*/
  CONSTANT(CS_USER_TYPE);

/*
** Define the bit values used in the format element of the CS_DATAFMT
** structure. The CS_FMT_JUSTIFY_RT flag exists for future use only.
*/
  CONSTANT(CS_FMT_UNUSED);
  CONSTANT(CS_FMT_NULLTERM);
  CONSTANT(CS_FMT_PADNULL);
  CONSTANT(CS_FMT_PADBLANK);
  CONSTANT(CS_FMT_JUSTIFY_RT);
  CONSTANT(CS_FMT_STRIPBLANKS);
  CONSTANT(CS_FMT_SAFESTR);

/*
** The following are bit values for the status field in the CS_DATAFMT
** structure.
** 
** CS_HIDDEN		Set if this column would normally be hidden
**			from the user.
**
** CS_KEY		Set if the column is a key column.
**
** CS_VERSION_KEY	Set if the column is part of the version key for
**			the row.
**
** CS_NODATA		Not currently used in Open Client/Server.
**
** CS_UPDATABLE		Set if the column is an updatable cursor column.
**
** CS_CANBENULL		Set if the column can contain NULL values.
**
** CS_DESCIN		
** CS_DESCOUT		Open Server-specific values for dynamic SQL.
**
** CS_INPUTVALUE	Set if the parameter is an input parameter value
**			for a Client-Library command.
**
** CS_UPDATECOL		Set if the parameter is the name of a column in
**			the update clause of a cursor declare command.
**
** CS_RETURN		Set if the parameter is a return parameter to
**			an RPC command.
**
** CS_TIMESTAMP		Set if the column is a timestamp column.
**
** CS_NODEFAULT		Open Server-specific status values for RPC parameters.
**
** CS_IDENTITY		Set if the column is an identity column.
**
*/
  CONSTANT(CS_HIDDEN);
  CONSTANT(CS_KEY);
  CONSTANT(CS_VERSION_KEY);
  CONSTANT(CS_NODATA);
  CONSTANT(CS_UPDATABLE);
  CONSTANT(CS_CANBENULL);
  CONSTANT(CS_DESCIN);
  CONSTANT(CS_DESCOUT);
  CONSTANT(CS_INPUTVALUE);
  CONSTANT(CS_UPDATECOL);
  CONSTANT(CS_RETURN);
  CONSTANT(CS_TIMESTAMP);
  CONSTANT(CS_NODEFAULT);
  CONSTANT(CS_IDENTITY);

/*
** The following are bit values for the status field in the CS_BROWSEDESC
** structure.
** 
** CS_EXPRESSION	Set when the column is the result of an expression.
**
** CS_RENAMED		Set when that the column's heading is not the 
**			original name of the column. 
*/
  CONSTANT(CS_EXPRESSION);
  CONSTANT(CS_RENAMED);

/*
** Define special precision/scale value for using the precision/scale of the
** source element when doing a conversion from a numeric/decimal datatype to a
** numeric/decimal datatype.
*/
  CONSTANT(CS_SRC_VALUE);

/*
** Minimum/maximum/default precision and scale values for numeric/decimal
** datatypes.
*/
  CONSTANT(CS_MIN_PREC);
  CONSTANT(CS_MAX_PREC);
  CONSTANT(CS_DEF_PREC);

  CONSTANT(CS_MIN_SCALE);
  CONSTANT(CS_MAX_SCALE);
  CONSTANT(CS_DEF_SCALE);

/*
** The datetime format for converting datetime to char.
*/
  CONSTANT(CS_DATES_SHORT);
  CONSTANT(CS_DATES_MDY1);
  CONSTANT(CS_DATES_YMD1);
  CONSTANT(CS_DATES_DMY1);
  CONSTANT(CS_DATES_DMY2);
  CONSTANT(CS_DATES_DMY3);
  CONSTANT(CS_DATES_DMY4);
  CONSTANT(CS_DATES_MDY2);
  CONSTANT(CS_DATES_HMS);
  CONSTANT(CS_DATES_LONG);
  CONSTANT(CS_DATES_MDY3);
  CONSTANT(CS_DATES_YMD2);
  CONSTANT(CS_DATES_YMD3);
  CONSTANT(CS_DATES_YDM1);
  CONSTANT(CS_DATES_MYD1);
  CONSTANT(CS_DATES_DYM1);
  CONSTANT(CS_DATES_SHORT_ALT);
  CONSTANT(CS_DATES_MDY1_YYYY);
  CONSTANT(CS_DATES_YMD1_YYYY);
  CONSTANT(CS_DATES_DMY1_YYYY);
  CONSTANT(CS_DATES_DMY2_YYYY);
  CONSTANT(CS_DATES_DMY3_YYYY);
  CONSTANT(CS_DATES_DMY4_YYYY);
  CONSTANT(CS_DATES_MDY2_YYYY);
  CONSTANT(CS_DATES_HMS_ALT);
  CONSTANT(CS_DATES_LONG_ALT);
  CONSTANT(CS_DATES_MDY3_YYYY);
  CONSTANT(CS_DATES_YMD2_YYYY);
  CONSTANT(CS_DATES_YMD3_YYYY);

/*
** Locale type information. CS_LC_MONETARY and CS_LC_NUMERIC are defined
** for future use.
*/
  CONSTANT(CS_LC_COLLATE);
  CONSTANT(CS_LC_CTYPE);
  CONSTANT(CS_LC_MESSAGE);
  CONSTANT(CS_LC_MONETARY);
  CONSTANT(CS_LC_NUMERIC);
  CONSTANT(CS_LC_TIME);
  CONSTANT(CS_LC_ALL);
  CONSTANT(CS_SYB_LANG);
  CONSTANT(CS_SYB_CHARSET);
  CONSTANT(CS_SYB_SORTORDER);
  CONSTANT(CS_SYB_COLLATE);
  CONSTANT(CS_SYB_LANG_CHARSET);
  CONSTANT(CS_SYB_TIME);
  CONSTANT(CS_SYB_MONETARY);
  CONSTANT(CS_SYB_NUMERIC);

/*
** Object type information for the cs_objects() API.
*/
  CONSTANT(CS_CONNECTNAME);
  CONSTANT(CS_CURSORNAME);
  CONSTANT(CS_STATEMENTNAME);
  CONSTANT(CS_CURRENT_CONNECTION);
  CONSTANT(CS_MIN_USERDATA);

/*
** Info type information for the ct_ds_objinfo() API.
*/
  CONSTANT(CS_DS_CLASSOID);
  CONSTANT(CS_DS_DIST_NAME);
  CONSTANT(CS_DS_NUMATTR);
  CONSTANT(CS_DS_ATTRIBUTE);
  CONSTANT(CS_DS_ATTRVALS);


/*
** The following are the possible values for cs_status field of
** CHAR_ATTRIB structure defined in intl_nls.h
*/

  CONSTANT(CS_STAT_DEFAULT);
  CONSTANT(CS_STAT_MULTIBYTE);
  CONSTANT(CS_STAT_SPACELAST);
  CONSTANT(CS_STAT_NONASCIISP);

/*
** Syntax identifier tokens for the CS_ATTRIBUTE union.
*/
  CONSTANT(CS_ATTR_SYNTAX_NOOP);
  CONSTANT(CS_ATTR_SYNTAX_STRING);
  CONSTANT(CS_ATTR_SYNTAX_INTEGER);
  CONSTANT(CS_ATTR_SYNTAX_BOOLEAN);
  CONSTANT(CS_ATTR_SYNTAX_ENUMERATION);
  CONSTANT(CS_ATTR_SYNTAX_TRANADDR);
  CONSTANT(CS_ATTR_SYNTAX_OID);


/*
** Predefined signal handlers for client and server signal handler
** libraries.
*/

  CONSTANT(CS_SIGNAL_IGNORE);
  CONSTANT(CS_SIGNAL_DEFAULT);
  CONSTANT(CS_ASYNC_RESTORE);
  CONSTANT(CS_SIGNAL_BLOCK);
  CONSTANT(CS_SIGNAL_UNBLOCK);


  return 0;
}

#define SY_OCCDF_ID "$Id: sy_occdf.c,v 1.2 1999/10/04 19:54:43 jim Exp $\n"
